#import "CircleLayout.h"
#import "CommonMacros.h"

#define ITEM_SIZE 100.0

@interface CircleLayout ()

@property (strong, nonatomic) NSMutableArray *updatingIndexPaths;

@end

@implementation CircleLayout

- (void)prepareLayout;
{
	[super prepareLayout];
	
	CGSize size = self.collectionView.frame.size;
	
	_cellCount = [self.collectionView numberOfItemsInSection:0];
	
	_center = CGPointMake(size.width / 2.0, size.height / 2.0);
	_radius = MIN(size.width, size.height) / 2.5;
}

- (CGSize)collectionViewContentSize;
{
	return self.collectionView.frame.size;
}

- (UICollectionViewLayoutAttributes *)layoutAttributesForItemAtIndexPath:(NSIndexPath *)path;
{
	UICollectionViewLayoutAttributes *attributes = [UICollectionViewLayoutAttributes 
	layoutAttributesForCellWithIndexPath:path];
	
	attributes.size = CGSizeMake(ITEM_SIZE, ITEM_SIZE);
	attributes.center = CGPointMake(_center.x + _radius * cosf(2 * path.item * M_PI / _cellCount),
									_center.y + _radius * sinf(2 * path.item * M_PI / _cellCount));
	
	return attributes;
}

- (NSArray *)layoutAttributesForElementsInRect:(CGRect)rect;
{
	NSMutableArray *attributes = [NSMutableArray array];

	for(NSInteger i = 0; i < self.cellCount; i++)
	{
		NSIndexPath *indexPath = [NSIndexPath indexPathForItem:i inSection:0];
		[attributes addObject:[self layoutAttributesForItemAtIndexPath:indexPath]];
	}
	
	return attributes;
}

- (void)prepareForCollectionViewUpdates:(NSArray *)updateItems
{
    self.updatingIndexPaths = [NSMutableArray arrayWithCapacity:[updateItems count]];
    
    for(UICollectionViewUpdateItem *item in updateItems)
    {
        if(item.updateAction == UICollectionUpdateActionDelete)
            [self.updatingIndexPaths addObject:[item indexPathBeforeUpdate]];
        else if(item.updateAction == UICollectionUpdateActionInsert)
            [self.updatingIndexPaths addObject:[item indexPathAfterUpdate]];
        else
            MERLogInfo(@"Other action: %d", item.updateAction);
    }
}

//- (UICollectionViewLayoutAttributes *)initialLayoutAttributesForInsertedItemAtIndexPath:(NSIndexPath *)itemIndexPath;
- (UICollectionViewLayoutAttributes *)initialLayoutAttributesForAppearingItemAtIndexPath:(NSIndexPath *)itemIndexPath;
{
	UICollectionViewLayoutAttributes *attributes = [self layoutAttributesForItemAtIndexPath:itemIndexPath];

    if([self.updatingIndexPaths containsObject:itemIndexPath])
    {
        [self.updatingIndexPaths removeAllObjects];
        
        attributes.alpha = 0.0;
        attributes.center = CGPointMake(_center.x, _center.y);
	}
    
	return attributes;
}

//- (UICollectionViewLayoutAttributes *)finalLayoutAttributesForDeletedItemAtIndexPath:(NSIndexPath *)itemIndexPath;
- (UICollectionViewLayoutAttributes *)finalLayoutAttributesForDisappearingItemAtIndexPath:(NSIndexPath *)itemIndexPath;
{
	UICollectionViewLayoutAttributes *attributes = [self layoutAttributesForItemAtIndexPath:itemIndexPath];

    if([self.updatingIndexPaths containsObject:itemIndexPath])
    {
        [self.updatingIndexPaths removeAllObjects];
        
        attributes.alpha = 0.0;
        attributes.center = CGPointMake(_center.x, _center.y);
        attributes.transform3D = CATransform3DMakeScale(0.1, 0.1, 1.0);
    }
    
	return attributes;
}


@end
