//
//  RootViewController.m
//  CollectionViewTest2
//
//  Created by Marc Respass on 10/01/2012.
//  Copyright (c) 2012 ILIOS Inc. All rights reserved.
//

#import "RootViewController.h"
#import "CommonMacros.h"
#import "CVBadgeCVC.h"
#import "JSBadgeView.h"

#import <QuartzCore/QuartzCore.h>
#include <AudioToolbox/AudioToolbox.h>

#define NORMAL_IMAGE_NAME_WHITE @"thumb-down-white"
#define NORMAL_IMAGE_NAME_GREY @"thumb-down-grey"
#define SELECTED_IMAGE_NAME_WHITE @"thumb-up-white"
#define SELECTED_IMAGE_NAME_GREY @"thumb-up-grey"

#define STANDARD_VIEW_WIDTH 100.0
#define STANDARD_VIEW_HEIGHT 100.0
#define CHOSEN_VIEW_WIDTH 100.0
#define CHOSEN_VIEW_HEIGHT 100.0

#define kChosenSection 0
#define kAvailableSection 1

#define DISABLED_KEY @"disabled"
#define LABEL_KEY @"label"
#define MAX_ITEMS 16

//static NSString *kSectionHeaderId = @"SectionHeaderIdentifier";

static NSString *kCellIdentifier = @"CellIdentifier";
static NSString *kEnabledIdentifier = @"EnabledIdentifier";
static NSString *kDisabledIdentifier = @"DisabledIdentifier";
static NSString *kChosenIdentifier = @"ChosenIdentifier";
static NSString *kHighlightedIdentifier = @"HighlightedIdentifier";

static SystemSoundID beep7;
static SystemSoundID beep8;

@interface RootViewController ()

@property NSMutableArray *availableItems;
@property NSMutableArray *chosenItems;

@end

@implementation RootViewController

- (id)init;
{
    if([super initWithNibName:NSStringFromClass([self class]) bundle:nil])
    {
        NSURL *soundURL = nil;
        
        soundURL = [[NSBundle mainBundle] URLForResource:@"beep-7" withExtension:@"wav"];
        AudioServicesCreateSystemSoundID((__bridge CFURLRef) soundURL, &beep7);
        soundURL  = [[NSBundle mainBundle] URLForResource:@"beep-8" withExtension:@"wav"];
        AudioServicesCreateSystemSoundID((__bridge CFURLRef) soundURL, &beep8);
        
        _chosenItems = [[NSMutableArray alloc] initWithCapacity:MAX_ITEMS];
        _availableItems = [[NSMutableArray alloc] initWithCapacity:MAX_ITEMS];
        for(int i = 0; i < MAX_ITEMS; i++)
        {
            NSString *label = [NSString stringWithFormat:@"%02d", (i + 1)];
            NSMutableDictionary *d = [NSMutableDictionary dictionaryWithObjectsAndKeys:label, LABEL_KEY, nil];
            [_availableItems addObject:d];
        }
    }
    return self;
}

#pragma mark - Init
- (void)dealloc;
{
    // call the following function when the sound is no longer used
    // (must be done AFTER the sound is done playing)
    AudioServicesDisposeSystemSoundID(beep7);
    AudioServicesDisposeSystemSoundID(beep8);
}

- (void)viewDidLoad
{
    MERLogDebug();

    [super viewDidLoad];

    UINib *nib = nil;
    
    nib = [UINib nibWithNibName:NSStringFromClass([CVBadgeCVC class]) bundle:nil];
    [self.collectionView registerNib:nib forCellWithReuseIdentifier:kCellIdentifier];
        
    UITapGestureRecognizer *tapRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleTapGesture:)];
    [self.collectionView addGestureRecognizer:tapRecognizer];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Cell Configuration Helpers
- (void)configureHighlightedCell:(CVBadgeCVC *)cell
{
    cell.badgeView.badgeTextColor = [UIColor darkGrayColor];
    cell.badgeView.badgeBackgroundColor = [UIColor yellowColor];
    cell.badgeView.badgeBorderColor = [UIColor redColor];
}

- (void)configureAvailableCell:(CVBadgeCVC *)cell
{
    cell.badgeView.badgeTextColor = [UIColor whiteColor];
    cell.badgeView.badgeBackgroundColor = [UIColor blueColor];
    cell.badgeView.badgeBorderColor = [UIColor greenColor];
}

#pragma mark - UICollectionViewDataSource
// The cell that is returned must be retrieved from a call to -dequeueReusableCellWithReuseIdentifier:forIndexPath:
- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath;
{
    CVBadgeCVC *cell = [collectionView dequeueReusableCellWithReuseIdentifier:kCellIdentifier forIndexPath:indexPath];

    NSDictionary *availableData = [self.availableItems objectAtIndex:indexPath.item];
    [self configureAvailableCell:cell];
    
    [cell setLabel:[availableData valueForKey:LABEL_KEY]];

    return cell;
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView;
{
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section;
{
    return [self.availableItems count];
}

#pragma mark - UICollectionViewDelegate
- (BOOL)collectionView:(UICollectionView *)collectionView shouldHighlightItemAtIndexPath:(NSIndexPath *)indexPath;
{
    if(indexPath.section == kAvailableSection)
    {
        NSDictionary *itemData = [self.availableItems objectAtIndex:indexPath.item];
        
        return [[itemData valueForKey:DISABLED_KEY] boolValue] == NO;
    }
    return YES;
}

- (void)collectionView:(UICollectionView *)collectionView didHighlightItemAtIndexPath:(NSIndexPath *)indexPath;
{
    MERLogDebug();
    CVBadgeCVC *cell = (CVBadgeCVC *)[collectionView cellForItemAtIndexPath:indexPath];
    
    [self configureHighlightedCell:cell];
}

- (void)collectionView:(UICollectionView *)collectionView didUnhighlightItemAtIndexPath:(NSIndexPath *)indexPath;
{
    MERLogDebug();
    CVBadgeCVC *cell = (CVBadgeCVC *)[collectionView cellForItemAtIndexPath:indexPath];
    
    [self configureAvailableCell:cell];

}

- (void)handleTapGesture:(UITapGestureRecognizer *)sender;
{
    if(sender.state == UIGestureRecognizerStateEnded)
    {
        CGPoint initialPinchPoint = [sender locationInView:self.collectionView];
        NSIndexPath *tappedCellPath = [self.collectionView indexPathForItemAtPoint:initialPinchPoint];
        
        if(tappedCellPath != nil)
        {
            if([self.availableItems count] <= 4)
                return;
            
            AudioServicesPlaySystemSound(beep7);

            [self.availableItems removeObjectAtIndex:tappedCellPath.item];
            
            [self.collectionView performBatchUpdates:^{
                [self.collectionView deleteItemsAtIndexPaths:[NSArray arrayWithObject:tappedCellPath]];
            } completion:NULL];
        }
        else
        {
            if([self.availableItems count] == MAX_ITEMS)
                return;

            AudioServicesPlaySystemSound(beep8);

            NSUInteger numItems = [self.availableItems count];
            NSString *label = [NSString stringWithFormat:@"%02d", numItems];
            NSMutableDictionary *newItem = [NSMutableDictionary dictionaryWithObjectsAndKeys:label, LABEL_KEY, nil];

            [self.availableItems addObject:newItem];
            
            [self.collectionView insertItemsAtIndexPaths:[NSArray arrayWithObject:
                                                          [NSIndexPath indexPathForItem:numItems inSection:0]]];
//            [self.collectionView performBatchUpdates:^{
//                [self.collectionView insertItemsAtIndexPaths:[NSArray arrayWithObject:
//                                                              [NSIndexPath indexPathForItem:numItems inSection:0]]];
//            } completion:NULL];
            
        }
    }
}

@end
